#ifndef _BABY_H_
#define _BABY_H_

#include "moving_entity.h"
#include "tile.h"
//#include "audiomixer.h"
#include <iostream>

class Baby : public MovingEntity {
	public:
	
	Baby (float x, float y) : MovingEntity(x, y), crying(false), cryDelay(0), previousFrame(0), cryFrame(0) {
		image = ImageMap::get("baby");
	}
	
	~Baby () {};
	
	void update () {
		updateY();
		
		// Make the baby face the player.
		Level* level = Level::current();	
		Entity* current = level->findEntity(x + 16, y + 16, this);
		Entity* behind = level->findEntity(x , y + 16, this);
		Entity* front = level->findEntity(x + 32, y + 16, this);
		
		if (!crying) {
			if (behind != 0 && behind->getType() == Tile::PLAYER) {
				facing = -1;
			} else if (front != 0 && front->getType() == Tile::PLAYER) {
				facing = 1;
			}
			
			// Check if the baby is underwater.
			if (current !=0 && (current->getType() == Tile::WATER || current->getType() == Tile::WATER_TOP)) {
				cry();
			}
			
		}
		
		if (crying && cryDelay <= 0) {
			switch(frame) {
				case 3: cryDelay = 15;
					frame++;
					break;

				case 4: cryDelay = 60;
					facing = 1;
					frame++;
					previousFrame = 4;
					break;
				
				case 5: cryDelay = 10;
					if (previousFrame == 6) {
						frame = 7;					
					} else {
						frame = 6;
					}
					previousFrame = 5;
					break;

				case 6: cryDelay = 10;
					frame = 5;
					previousFrame = 6;
					break;

				case 7: cryDelay = 10;
					frame = 5;
					previousFrame = 7;
					break;
			}

			if (frame >= 5) {
				cryFrame = (cryFrame + 1) % 4;
			}
			
		}
		if (cryDelay >=1) {
			cryDelay--;
		}
	};

	void draw () const {
		MovingEntity::draw();
		if (crying && frame >= 5 && previousFrame != 4) {
			Image* tears = ImageMap::get("cry");
			tears->drawFrame(cryFrame, x, y);
		}
	};

	bool isCrying() { return crying; };

	int offsetL () const { return 10; }
	int offsetR () const { return 22; }
	int offsetT () const { return (frame >= 5) ? 14 : 10; }
	int offsetB () const { return 32; }

	void cry() {
		if (!crying) {
			frame = 3;
			cryFrame = 0;
			crying = true;
			Level* level = Level::current();
			level->babiesAlive--;
			//AudioMixer::play("phaser");
		}
	};

	int getLayer() const { return Tile::MIDDLEGROUND; };	
	int getType() const { return Tile::BABY; };

	void standingOn (Entity* entity) {
		if (vy >= 12.0f) {
			cry();
		}
		
		MovingEntity::standingOn(entity);
	}

	void stoodOnBy (MovingEntity* entity, float speed) {
		if (entity->getType() != Tile::PLAYER) {
			cry();
		}
	}

	private:
	bool crying;
	int cryDelay;
	int previousFrame;
	int cryFrame;
};

#endif

