#include "level.h"
#include "bullet.h"
#include "imagemap.h"
#include "moving_entity.h"
#include "main.h"
#include "tile.h"

#include <boost/foreach.hpp>
#include <iostream>

Level* Level::s_activeLevel = 0;

Level::Level (int tileData[WIDTH][HEIGHT]) : babiesAlive(0), player(0), endDelay(240) {
	s_activeLevel = this;
	
	// initialise
	for (int x = 0; x < WIDTH; x++) {
		for (int y = 0; y < HEIGHT; y++) {
			int type = tileData[x][y];
			
			if (type == Tile::BABY) {
				babiesAlive++;
			}
			
			MovingEntity* entity = MovingEntity::create(this, type, x, y);
			
			if (entity) {
				entities.push_back(entity);
				
				tiles[x][y] = new Tile(0, x, y);
			} else {
				tiles[x][y] = new Tile(type, x, y);
			}
		}
	}
	
	// If there are no babies in the level, allow the player to mess around.	
	if (!babiesAlive) {
		babiesAlive++;
	}
}

Level::~Level () {
	BOOST_FOREACH(MovingEntity* entity, entities) {
		delete entity;
	}
	
	for (int x = 0; x < WIDTH; x++) {
		for (int y = 0; y < HEIGHT; y++) {
			delete tiles[x][y];
		}
	}
}

Entity* Level::findEntity (int x, int y, Entity* asker) {
	int ix = x / Tile::WIDTH;
	int iy = y / Tile::HEIGHT;
	
	Entity* tile = getTile(ix, iy);
	
	if (tile->getType() != 0 && tile->contains(x, y)) {
		return tile;
	}
	
	// hack for double size tiles: only fridge?
	
	tile = getTile(ix, iy - 1);
	
	if (tile->getType() != 0 && tile->contains(x, y)) {
		return tile;
	}
	
	// look for moving entities
	
	BOOST_FOREACH(MovingEntity* entity, entities) {
		if (entity == asker) {
			continue;
		}
		
		if (entity->contains(x, y)) {
			return entity;
		}
	}
	
	return 0;
}

void Level::handleInput (SDL_Event& event) {
	switch (event.type) {
		case SDL_KEYDOWN:
			keyDown(event);
		break;
		case SDL_QUIT:
			exit(0);
		break;
	}
}

void Level::keyDown (SDL_Event& event) {
	switch (event.key.keysym.sym) {
		case SDLK_ESCAPE:
			transitionTo(titleScreen);
		break;
		case SDLK_r:
			reset();
		break;
		case SDLK_n:
			nextLevel();
		break;
		default: {}
	}
}

void Level::update () {
	BOOST_FOREACH(MovingEntity* entity, entities) {
		entity->update();
	}
	
	Bullet::updateAll();
	
	if (babiesAlive == 0) {
		endDelay--;
	}
	
	if (endDelay == 0) {
		nextLevel();
	}
}

void Level::draw () const {
	ImageMap::get("background")->draw(0, 0);
	
	draw(Tile::BACKGROUND);
	draw(Tile::MIDDLEGROUND);
	draw(Tile::FOREGROUND);	
}

void Level::draw (int layer) const {
	for (int x = 0; x < WIDTH; x++) {
		for (int y = 0; y < HEIGHT; y++) {
			if (tiles[x][y]->getLayer() == layer) {
				tiles[x][y]->draw();
			}
		}
	}
	
	BOOST_FOREACH(MovingEntity* entity, entities) {
		if (entity->getLayer() == layer) {
			entity->draw();
		}
	}
	
	if (layer == Tile::MIDDLEGROUND) {
		Bullet::drawAll();
	}
	
}

