#include "moving_entity.h"
#include "level.h"

#include "player.h"
#include "baby.h"
#include "parent.h"
#include "moving_block.h"

const float MovingEntity::GRAVITY = 0.6f;

MovingEntity* MovingEntity::create (Level* level, int tileID, int ix, int iy) {
	int x = ix * Tile::WIDTH;
	int y = iy * Tile::HEIGHT;
	switch (tileID) {
		case Tile::PLAYER:
			return new Player(x, y);
		case Tile::BABY:
			return new Baby(x, y);
		case Tile::MUM:
		case Tile::DAD:
			return new Parent(tileID, x, y);
		case Tile::MOVING_BLOCK:
			return new MovingBlock(tileID, x, y);
		default:
			return 0;
	}
}

#define COLLISION_TEST(x, y, method) entity = level->findEntity(x, y, this); \
	if (entity != 0 && entity->solid()) { method(entity); }

void MovingEntity::checkCollisionsLR () {
	Level* level = Level::current();
	
	Entity* entity;
	
	int y1 = t() + 1;
	int y2 = b() - 1;
	int y3 = (y1 + y2) / 2;
	
	COLLISION_TEST(l(), y1, pushingLeft);
	COLLISION_TEST(l(), y2, pushingLeft);
	COLLISION_TEST(l(), y3, pushingLeft);
	
	COLLISION_TEST(r(), y1, pushingRight);
	COLLISION_TEST(r(), y2, pushingRight);
	COLLISION_TEST(r(), y3, pushingRight);
}

void MovingEntity::checkCollisionsUD () {
	Level* level = Level::current();
	
	Entity* entity;
	
	int x1 = l() + 1;
	int x2 = r() - 1;
	int x3 = (x1 + x2) / 2;
	
	COLLISION_TEST(x1, t(), bangingHead);
	COLLISION_TEST(x2, t(), bangingHead);
	COLLISION_TEST(x3, t(), bangingHead);
	
	COLLISION_TEST(x1, b(), standingOn);
	COLLISION_TEST(x2, b(), standingOn);
	COLLISION_TEST(x3, b(), standingOn);
}

void MovingEntity::standingOn (Entity* entity) {
	if (entity->squashable()) {
		static_cast<MovingEntity*>(entity)->stoodOnBy(this, vy);
	}
	
	vy = 0;
	
	y = entity->t() - offsetB();
}

void MovingEntity::pushingLeft (Entity* entity) {
	x = entity->r() - offsetL();
	
	tryPush(entity, -1);
}

void MovingEntity::pushingRight (Entity* entity) {
	x = entity->l() - offsetR();
	
	tryPush(entity, 1);
}

void MovingEntity::tryPush (Entity* e, int dir) {
	if (e->pushable()) {
		MovingEntity* entity = static_cast<MovingEntity*>(e);
		
		// I would to have variable speed pushing, but it breaks things
		// if speed is not an integer.
		float speed = dir;
		
		x -= speed;
		entity->x += speed;
		
		entity->checkCollisionsLR();
	}
}

void MovingEntity::bangingHead (Entity* entity) {
	// Magic two
	y = entity->b() - offsetT() + 2;
}

void MovingEntity::updateX (float dist) {
	if (dist == 0.0f) { return; }
	
	facing = (dist > 0.0f) ? 1 : -1;
	
	x += dist;
	
	checkCollisionsLR();
}

void MovingEntity::updateY () {
	y += vy + 0.5f * GRAVITY;
	
	vy += GRAVITY;
	
	checkCollisionsUD();
}

void MovingEntity::update () {
	updateY();
}

void MovingEntity::draw () const {
	if (facing == 1) {
		image->drawFrame(frame, x, y);
	} else {
		image->drawFrame(frame, x, y, true);
	}
}

