#ifndef _MOVING_ENTITY_H_
#define _MOVING_ENTITY_H_

#include "entity.h"

class Level;

class MovingEntity : public Entity {
	public:
	
	static const float GRAVITY;
	
	static MovingEntity* create (Level* level, int tileID, int x, int y);
	
	MovingEntity (float x, float y) : image(0), x(x), y(y), vy(0), frame(0), facing(1){}
	virtual ~MovingEntity () {}
	
	void checkCollisionsLR ();
	void checkCollisionsUD ();
	
	virtual void update ();
	virtual void draw () const;
	
	// Move this distance in the x axis
	void updateX(float dist);
	
	// Move in the y axis according to gravity
	void updateY();
	
	virtual int l () const { return x + offsetL(); }
	virtual int r () const { return x + offsetR(); }
	virtual int t () const { return y + offsetT(); }
	virtual int b () const { return y + offsetB(); }
	
	virtual bool solid () const { return true; }

	virtual int getLayer () const = 0;
	virtual int getType () const = 0;
	
	virtual bool squashable () const { return true; }
	virtual bool pushable () const { return true; }
	
	virtual void standingOn (Entity* entity);
	virtual void pushingLeft (Entity* entity);
	virtual void pushingRight (Entity* entity);
	virtual void bangingHead (Entity* entity);
	
	void tryPush (Entity* entity, int direction);
	
	// Notification methods
	virtual void stoodOnBy (MovingEntity* entity, float speed) {}
	virtual void pushedBy (MovingEntity* entity, int direction) {}
	
	protected:
	
	Image* image;
	
	float x;
	float y;
	
	float vy;
	
	int frame;

	int facing;
};

#endif

