#ifndef _TILE_H_
#define _TILE_H_

#include "entity.h"
#include "imagemap.h"

class Tile : public Entity {
	public:
	
	struct TileInfo {
		int width;
		int height;
		int layer;
	};
	
	// Tile dimensions.
	static const int WIDTH = 32;
	static const int HEIGHT = 32;
	
	// Tile types.
	static const int BLANK = 0;
	static const int SOLID = 1;
	static const int FRIDGE = 2;
	static const int TV = 3;
	static const int PLAYER = 4;
	static const int BABY = 5;
	static const int BAT = 6;
	static const int SLINGSHOT = 7;
	static const int MUM = 8;
	static const int DAD = 9;
	static const int MOVING_BLOCK = 10;
	static const int WATER_TOP = 11;
	static const int WATER = 12;
	
	// Available layers.
	static const int BACKGROUND = -1;
	static const int MIDDLEGROUND = 0;
	static const int FOREGROUND = 1;
	
	static TileInfo tileInfo [13];
	static void initialiseTileInfo();
	
	int type;
	const int ix;
	const int iy;
	
	Tile (int typeID, int ix, int iy) : type(typeID), ix(ix), iy(iy) { if (type >= 13 || type < 0) { type = 0; } }
	
	void update () {}
	void draw () const { ImageMap::get("tiles")->drawFrame(type, ix * WIDTH, iy * HEIGHT); }
	
	bool solid () const { return type > 0 && type < BAT; }
	
	int l () const { return ix * WIDTH + offsetL(); }
	int r () const { return ix * WIDTH + offsetR(); }
	int t () const { return iy * HEIGHT + offsetT(); }
	int b () const { return iy * HEIGHT + offsetB(); }

	int offsetL () const { return 0; }
	int offsetR () const { return WIDTH; }
	int offsetT () const { return 0; }
	int offsetB () const { return (type == FRIDGE) ? 2 * HEIGHT - 1 : HEIGHT - 1; }

	int getLayer() const { return tileInfo[type].layer; }
	int getType() const { return type; }
	
	virtual bool squashable () const { return false; }
	virtual bool pushable () const { return false; }
};

#endif

