#include "window.h"

#include <cstdlib> // For some useful functions such as atexit :)
#include <iostream>

using namespace std;

bool Window::s_ready = false;

int Window::s_width = 800;
int Window::s_height = 640;

int Window::s_fullscreen_width;
int Window::s_fullscreen_height;

int Window::s_old_width;
int Window::s_old_height;

bool Window::s_resizable = false;
bool Window::s_fullscreen = false;

SDL_Surface* Window::s_screen;

bool Window::init (const int w, const int h) {
	s_width = w;
	s_height = h;
	
	return init();
}

bool Window::init (const int w, const int h, const char* title) {
	// init needs to be called before setTitle
	bool retVal = init(w, h);
	
	setTitle(title);
	
	return retVal;
}

bool Window::init () {

	// Centre the window.
	putenv("SDL_VIDEO_WINDOW_POS");
	putenv("SDL_VIDEO_CENTERED=1");
	getenv("SDL_VIDEO_WINDOW_POS");
	getenv("SDL_VIDEO_CENTERED");

	if ( SDL_Init(SDL_INIT_VIDEO) != 0 ) {
		fprintf(stderr, "Unable to initialize SDL: %s\n", SDL_GetError());
		return false;
	}

	atexit(SDL_Quit); // Clean it up nicely :)
	
	const SDL_VideoInfo* videoInfo = SDL_GetVideoInfo();
	
	s_fullscreen_width = videoInfo->current_w;
	s_fullscreen_height = videoInfo->current_h;
	
	s_ready = true;

	return reset();
}

bool Window::reset () {
	
	if (! s_ready) {
		return false;
	}
	
	Uint32 flags = SDL_ANYFORMAT;
	
	if (s_resizable) {flags |= SDL_RESIZABLE; }
	if (s_fullscreen) {flags |= SDL_FULLSCREEN; }
	
	s_screen = SDL_SetVideoMode(s_width, s_height, 32, flags);
	
	if (s_screen == NULL) {
		fprintf(stderr, "Unable to set video mode: %s\n", SDL_GetError());
		return false;
	}
	
	return true;
}

bool Window::setTitle (const char* title) {
	if (! s_ready) { return false; }
	
	SDL_WM_SetCaption(title, title);
	
	return true;
}

void Window::setWidth (const int w) {
	s_width = w;
	
	reset();
}

void Window::setHeight (const int h) {
	s_height = h;
	
	reset();
}

void Window::resize (const int w, const int h) {
	s_width = w;
	s_height = h;
	
	reset();
}

void Window::setResizable (bool b) {
	s_resizable = b;
	
	reset();
}

void Window::setFullscreen (bool b) {
	s_fullscreen = b;
	
	if (s_fullscreen) {
		s_old_width = s_width;
		s_old_height = s_height;
		
		if (s_resizable) {
			s_width = s_fullscreen_width;
			s_height = s_fullscreen_height;
		}
	} else if (s_resizable) {
		s_width = s_old_width;
		s_height = s_old_height;
	}
	
	reset();
}

void Window::toggleFullscreen () {
	setFullscreen(! s_fullscreen);
}

